_cache = {}


def add(key: str, value) -> None:
    _cache[key] = value


def get(key: str):
    return _cache[key]


def exists(key: str) -> bool:
    return key in _cache


def clear() -> None:
    _cache.clear()



from polylith.test import report
from polylith.test.core import get_brick_imports_in_tests, get_changed_files
from polylith.test.tests import create_test

__all__ = ["report", "create_test", "get_brick_imports_in_tests", "get_changed_files"]



from pathlib import Path

from cleo.helpers import option
from poetry.console.commands.command import Command
from polylith import commands, configuration, diff, repo
from polylith.poetry.commands.diff import command_options


class TestDiffCommand(Command):
    name = "poly test diff"
    description = (
        "Shows the Polylith projects and bricks that are affected by changes in tests."
    )

    options = command_options + [
        option(
            long_name="bricks",
            description="Bricks affected by changes in tests",
            flag=True,
        ),
        option(
            long_name="projects",
            description="Projects affected by changes in tests",
            flag=True,
        ),
    ]

    def handle(self) -> int:
        since = self.option("since")

        options = {
            "short": self.option("short"),
            "bricks": self.option("bricks"),
            "projects": self.option("projects"),
        }

        root = repo.get_workspace_root(Path.cwd())
        ns = configuration.get_namespace_from_config(root)

        tag = diff.collect.get_latest_tag(root, since) or since

        if tag:
            commands.test.run(root, ns, tag, options)
        else:
            self.line("No matching tags or commits found in repository.")

        return 0



from pathlib import Path
from typing import List, Union

from polylith import check, diff, imports


def is_test(root: Path, ns: str, path: Path, theme: str) -> bool:
    expected = "test"
    file_path = path.as_posix()

    if theme == "loose":
        test_path = Path(root / f"{expected}/").as_posix()

        return str.startswith(file_path, test_path)

    return f"/{expected}/{ns}" in file_path


def get_changed_files(root: Path, tag_name: Union[str, None]) -> List[Path]:
    tag = diff.collect.get_latest_tag(root, tag_name) or tag_name

    if not tag:
        return []

    return [root / f for f in diff.collect.get_files(tag)]


def get_brick_imports_in_tests(
    root: Path, ns: str, theme: str, files: List[Path]
) -> dict:
    matched = {f for f in files if is_test(root, ns, f, theme)}

    listed_imports = [imports.list_imports(m) for m in matched]

    all_imports = {k: v for k, v in enumerate(listed_imports)}

    return check.grouping.extract_brick_imports(all_imports, ns)



from typing import List, Set

from polylith import info
from polylith.reporting import theme
from rich.console import Console
from rich.padding import Padding


def print_report_summary(
    projects_data: List[dict], bases: Set[str], components: Set[str], tag: str
) -> None:
    console = Console(theme=theme.poly_theme)

    number_of_projects = len(projects_data)
    number_of_components = len(components)
    number_of_bases = len(bases)

    console.print(Padding("[data]Projects and bricks affected by changes in tests[/]", (1, 0, 0, 0)))
    console.print(Padding(f"[data]Test diff based on {tag}[/]", (0, 0, 1, 0)))

    console.print(f"[proj]Affected projects[/]: [data]{number_of_projects}[/]")
    console.print(f"[comp]Affected components[/]: [data]{number_of_components}[/]")
    console.print(f"[base]Affected bases[/]: [data]{number_of_bases}[/]")


def print_detected_changes(changes: List[str], options: dict):
    short = options.get("short", False)

    if not changes:
        return

    console = Console(theme=theme.poly_theme)

    if short:
        console.out(",".join(changes))
        return

    for item in changes:
        console.print(f"[data]:gear: Changes affecting [/][data]{item}[/]")


def print_projects_affected_by_changes(
    projects_data: List[dict], options: dict
) -> None:
    sorted_projects = [p["path"].name for p in projects_data]

    print_detected_changes(sorted_projects, options)


def print_detected_changes_affecting_bricks(
    bases: Set[str], components: Set[str], options: dict
) -> None:
    bricks = bases.union(components)
    changes = sorted(list(bricks))

    print_detected_changes(changes, options)


def print_test_report(
    projects_data: List[dict], bases: Set[str], components: Set[str], options: dict
) -> None:
    short = options.get("short", False)

    b = sorted(list(bases))
    c = sorted(list(components))

    if not b and not c:
        return

    if short:
        info.print_compressed_view_for_bricks_in_projects(projects_data, b, c)
    else:
        info.print_bricks_in_projects(projects_data, b, c)



from pathlib import Path
from typing import List, Set, Tuple, Union

from polylith import bricks, configuration, diff, info, test


def get_imported_bricks_in_tests(
    root: Path, ns: str, tag_name: Union[str, None], theme: str
) -> Set[str]:
    files = test.get_changed_files(root, tag_name)
    brick_imports = test.get_brick_imports_in_tests(root, ns, theme, files)

    return set().union(*brick_imports.values())


def extract_brick_names(bricks_data: List[dict], imported_bricks: Set[str]) -> Set[str]:
    return {v for b in bricks_data for v in b.values() if v in imported_bricks}


def get_affected_bricks(
    root: Path, ns: str, tag_name: Union[str, None], theme: str
) -> Tuple[Set[str], Set[str]]:
    found = get_imported_bricks_in_tests(root, ns, tag_name, theme)

    bases = extract_brick_names(bricks.get_bases_data(root, ns), found)
    components = extract_brick_names(bricks.get_components_data(root, ns), found)

    return bases, components


def get_affected_projects(
    root: Path, ns: str, bases: Set[str], components: Set[str]
) -> List[dict]:
    projects_data = [p for p in info.get_projects_data(root, ns) if info.is_project(p)]

    names = diff.collect.get_projects_affected_by_changes(
        projects_data, [], list(bases), list(components)
    )

    return [p for p in projects_data if p["path"].name in names]


def run(root: Path, ns: str, tag: str, options: dict) -> None:
    theme = configuration.get_theme_from_config(root)

    bases, components = get_affected_bricks(root, ns, tag, theme)
    projects_data = get_affected_projects(root, ns, bases, components)

    if options.get("bricks"):
        test.report.print_detected_changes_affecting_bricks(bases, components, options)
        return

    if options.get("projects"):
        test.report.print_projects_affected_by_changes(projects_data, options)
        return

    test.report.print_report_summary(projects_data, bases, components, tag)
    test.report.print_test_report(projects_data, bases, components, options)



from functools import lru_cache
from pathlib import Path

import yaml


@lru_cache
def load_yaml(path: Path) -> dict:
    with open(path, "rb") as f:
        try:
            return yaml.safe_load(f)
        except yaml.YAMLError as e:
            raise ValueError(f"Failed loading {path}: {repr(e)}") from e
